/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.tools.AOutputForwardRunnable.java
 * Created on: Dec 1, 2016
 */
package net.bigeon.gclc.utils;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

/** An incomplete implematation used to forward messages from a piped console.
 * <p>
 * This forwarding can be interrupted without closing the piped manager.
 *
 * @author Emmanuel Bigeon */
public abstract class AOutputForwardRunnable implements Runnable {

    /** The class logger. */
    private static final Logger      LOGGER          = Logger
            .getLogger(AOutputForwardRunnable.class.getName());
    /** The default timeout (one tenth of second). */
    private static final long        DEFAULT_TIMEOUT = 100;
    /** The manager. */
    protected final PipedConsoleOutput manager;
    /** The timeout. */
    protected final long               timeout;

    /** Create a forwarding runnable.
     *
     * @param manager the manager */
    public AOutputForwardRunnable(final PipedConsoleOutput manager) {
        super();
        this.manager = manager;
        timeout = DEFAULT_TIMEOUT;
    }

    /** Create a forward runnable with the given timeout.
     * <p>
     * Short timeout will be very responsive to the application actual messages, but
     * may use computation time if the application is not verbose. Long timeout will
     * save computation time, but will read batches of messages at once if the
     * application is verbose. The right length for the timeout is likely to depend
     * on the application and the use of it.
     * <p>
     * If you do not know what timeout length to use, please use the
     * {@link #AOutputForwardRunnable(PipedConsoleOutput)} constructor.
     *
     * @param manager the manager
     * @param timeout the timeout between message requests. */
    public AOutputForwardRunnable(final PipedConsoleOutput manager, final long timeout) {
        super();
        this.manager = manager;
        this.timeout = timeout;
    }

    /** Do forward the line.
     *
     * @param m the line to forward */
    protected abstract void forwardLine(String m);

    /** Test if the runable is still running.
     *
     * @return if the thread should keep running */
    protected abstract boolean isRunning();

    /* (non-Javadoc)
     * @see java.lang.Runnable#run() */
    @Override
    public final void run() {
        try {
            while (isRunning()) {
                while (isRunning() && !manager.available()) {
                    waitASec();
                }
                if (!isRunning()) {
                    return;
                }
                final String m = manager.readNextLine();
                forwardLine(m);
            }
        } catch (final IOException e) {
            LOGGER.log(Level.SEVERE, "Unexpected problem in manager", //$NON-NLS-1$
                    e);
        }
    }

    /** a method to wait some time. */
    protected final void waitASec() {
        try {
            synchronized (this) {
                wait(timeout);
            }
        } catch (final InterruptedException e) {
            LOGGER.log(Level.SEVERE, "Interrupted wait", //$NON-NLS-1$
                    e);
            Thread.currentThread().interrupt();
        }
    }

}
